import json

from groq import AsyncGroq

from src.core.ai_services.schemas import MessageFromChatSchema, ResponseFromAiSchema
from .constants import GROQ_BASE_MESSAGE
from ..base import BaseAiService
from ...settings.base import settings


class GroqHelper(BaseAiService):
    def __init__(
        self,
        api_key: str,
        model: str,
        temperature: int,
    ) -> None:
        self.api_key = api_key
        self.model = model
        self.temperature = temperature

        self._client = AsyncGroq(
            api_key=self.api_key,
        )

    @staticmethod
    def _serialize_messages_to_promt(
        messages: list[MessageFromChatSchema],
    ) -> dict:
        messages_for_request = GROQ_BASE_MESSAGE.copy()
        extend_message = {
            "role": "user",
            "content": json.dumps(
                {
                    "messages": [
                        message.model_dump_with_datetime() for message in messages
                    ]
                }
            ),
        }

        messages_for_request.append(extend_message)

        return messages_for_request

    @staticmethod
    def _serialize_response_to_json(
        response_text: str,
    ) -> ResponseFromAiSchema:
        print(response_text)
        print(len(response_text))
        print("groq")
        response = response_text.replace('\n', '')
        response_as_dict = json.loads(response)

        return ResponseFromAiSchema(**response_as_dict)

    async def create_request_ai(
        self,
        messages: list[MessageFromChatSchema],
    ) -> ResponseFromAiSchema:
        messages_for_promt = self._serialize_messages_to_promt(messages)
        response = await self._client.chat.completions.create(
            messages=messages_for_promt,
            model=self.model,
            temperature=self.temperature,
        )

        return self._serialize_response_to_json(
            response.choices[0].message.content,
        )


groq_helper = GroqHelper(
    api_key=settings.GROQ.API_KEY,
    model=settings.GROQ.MODEL,
    temperature=settings.GROQ.TEMPERATURE,
)
