from sqlalchemy import insert
from sqlalchemy.ext.asyncio import AsyncSession

from src.core.ai_services.schemas import MessageFromChatSchema
from src.core.database import User, TgChat, TgMessage


async def create_new_user(
        session: AsyncSession,
        **kwargs
) -> User:
    new_user = User(**kwargs)

    session.add(new_user)
    await session.commit()
    return new_user


async def create_new_chat(
        session: AsyncSession,
        **kwargs
) -> TgChat:
    new_chat = TgChat(**kwargs)
    session.add(new_chat)
    await session.commit()

    return new_chat


async def bulk_insert_messages(
        messages: list[MessageFromChatSchema],
        session: AsyncSession,
) -> None:
    stmt = (
        insert(TgMessage),
        [
            *[message.model_dump() for message in messages],
        ]
    )
    await session.execute(*stmt)
    await session.commit()

