from pyrogram.types.user_and_chats import User, Chat

from src.core.tg_service.schemas import UserFromMessageSchema
from src.core.database.connect import db_helper
from src.core.redis_helper.redis_connect import redis_client
from src.core.tg_service import crud as tg_crud


async def check_user_exists(
        user_pyrogram: User,
) -> None:
    user_schema = UserFromMessageSchema(
        username=user_pyrogram.username,
        first_name=user_pyrogram.first_name,
        last_name=user_pyrogram.last_name,
        id=user_pyrogram.id,
    )

    user_model = await redis_client.get(name=str(user_schema.id))
    if user_model:
        return

    async with db_helper.get_async_session() as session:
        user_model = await session.get(id=user_model.id)

        if not user_model:
            await tg_crud.create_new_user(
                session=session,
                **user_schema.model_dump()
            )

    await redis_client.set(name=str(user_schema.id), value=True)


async def check_chat_exists(
        chat_pyrogram: Chat
) -> None:
    chat = await redis_client.get(chat_id=str(chat_pyrogram.id))

    if chat:
        return

    async with db_helper.get_async_session() as session:
        chat = await session.get(id=chat_pyrogram.id)

        if not chat:
            await tg_crud.create_new_chat(
                session=session,
                id=chat_pyrogram.id,
                chat_type=chat_pyrogram.type.value,
                title=chat_pyrogram.title,
            )
    await redis_client.set(name=str(chat_pyrogram.id), value=True)


